#!/usr/bin/env gosh
;;; coding: euc-jp
;;; -*- scheme -*-
;;; vim:set ft=scheme ts=8 sts=2 sw=2 et:
;;; $Id: tcpcgi-server-kickstart.scm 211 2005-10-01 21:34:51Z nekoie $

(add-load-path "./lib")
(add-load-path ".")
(use tcpcgi-server)

(use gauche.process)
(use gauche.threads)
(use srfi-1)
(use srfi-13)
(use srfi-19)
(use text.tree)
(use text.html-lite)
(use util.match)
(use www.cgi)

(define *current* 0)

(define *mutex* (make-mutex))
(define *cond* (make-condition-variable))
(define *owner* 'page)

(define *timestamp* (time->seconds (current-time)))

(define *content-file* #f)

(define *title* "")

(define (touch)
  (mutex-lock! *mutex*)
  (set! *timestamp* (time->seconds (current-time)))
  (condition-variable-broadcast! *cond*)
  (mutex-unlock! *mutex*))

(define (prompt)
  (format (current-output-port) "[[~d]]> " *current*)
  (flush))

(define (command-thunk)
  (prompt)
  (let loop ()
    (let1 line (read-line)
      (when (eof-object? line) (exit 0))
      (cond
       ((string=? line "-")
        (dec! *current*)
        (when (< *current* 0) (set! *current* 0)))
       ((string=? line "l")
        (load *content-file*))
       (else
        (inc! *current*)
        (when (>= *current* (length *pages*)) (set! *current* 0))))
      (touch)
      (prompt)
      (loop))))

(define (top-cgi)
  (write-tree
   (list
    (cgi-header :content-type "text/html; charset=utf-8")
    "<html>
<head>
<title>"*title*"</title>
<script type=\"text/javascript\"><!--
var count = 0
var timestamp = 0
function pull() {
  xmlhttp = new XMLHttpRequest();
  if (xmlhttp) {
    xmlhttp.onreadystatechange = pullCB;
    xmlhttp.open('GET', 'page?' + count++ + '&ts=' + timestamp, true);
    xmlhttp.send(null);
  }
}

function pullCB() {
  try {
    if (xmlhttp.readyState == 4 && xmlhttp.status == 200) {
      if (xmlhttp.responseText != null) {
        timestamp = xmlhttp.getResponseHeader('x-timestamp')
        document.getElementById('disp').innerHTML = xmlhttp.responseText;
      }
      pull()
    }
  } catch (error) {
    alert('xmlhttp error: connection reset.  please reload the page.');
  }
}

function req(s) {
  xmlhttp2 = new XMLHttpRequest();
  if (xmlhttp2) {
    xmlhttp2.open('GET', s + '?' + count++, true);
    xmlhttp2.send(null);
  }
  return false
}

// --></script>
</head>
<body onload='pull()' style='width: 720px; height:560px; margin: 0; padding: 0; font-family: sans-serif; background-image: url(http://schemearts.com/images/bg.jpg)'>
<div style='text-align:right; font-size:15px; font-family: serif'>
<a onclick='return req(\"start\")' href='.'></a>
<a onclick='return req(\"prev\")' href='.'></a>
<a onclick='return req(\"refresh\")' href='.'></a>
<a onclick='return req(\"next\")' href='.'></a>
<a onclick='return req(\"end\")' href='.'></a>
</div>
<div id='disp'></div>
</body>
</html>")))

(define (page-cgi)
  (let* ((params (cgi-parse-parameters))
         (pts (cgi-get-parameter "ts" params :default 0 :convert x->number)))
    (let loop ()
      (mutex-lock! *mutex*)
      (let1 ts *timestamp*
        (cond
         ((> ts pts)
          (mutex-unlock! *mutex*)
          (write-tree
           (list
            (cgi-header :content-type "text/plain; charset=utf-8"
                        :x-timestamp (x->string ts))
            (list-ref *pages* *current* '()))))
         (else
          (mutex-unlock! *mutex* *cond*)
          (loop)))))))

(define (next-cgi)
  (inc! *current*)
  (when (>= *current* (length *pages*))
    (set! *current* (- (length *pages*) 1)))
  (touch)
  (write-tree (cgi-header)))

(define (prev-cgi)
  (dec! *current*)
  (when (< *current* 0) (set! *current* 0))
  (touch)
  (write-tree (cgi-header)))

(define (start-cgi)
  (set! *current* 0)
  (touch)
  (write-tree (cgi-header)))

(define (end-cgi)
  (set! *current* (- (length *pages*) 1))
  (touch)
  (write-tree (cgi-header)))

(define (refresh-cgi)
  (touch)
  (write-tree (cgi-header)))

(define (error404-cgi)
  (write-tree
   (list
    (cgi-header :content-type "text/html; charset=utf-8")
    (html:html
     (html:head
      (html:title "404 Not found"))
     (html:body
      (html:h1 "404 Not found"))))))

;; alistǻꤹ롣
(define *path-dispatch*
  `(
    ("/" ,top-cgi)
    ("/start" ,start-cgi)
    ("/page" ,page-cgi)
    ("/next" ,next-cgi)
    ("/prev" ,prev-cgi)
    ("/end"  ,end-cgi)
    ("/refresh" ,refresh-cgi)
    ))

(define *tcpcgi-server*
  (make <tcpcgi-server>
    :server-addr "0.0.0.0"
    :server-port 8888 ; 801024portȤˤroot¤ɬס
    ;:server-name "hoge.com"
    :max-clients 4
    ;:child-uid "nobody"
    ;:child-gid "nobody"
    ;; ĤrootǼ¹ԤƤΤͭ
    :connection-log-port (current-error-port)

    ;; ʲΥѥ᡼<tcpcgi>Ʊ

    ;; ʲ:*-dispatchϡν˽롣
    ;;:vhost-dispatch *vhost-dispatch* ; vhostȤʤʤ
    ;; vhostΤɤˤޥåʤäϼpath-dispatch¹Ԥ롣
    :path-dispatch *path-dispatch*
    ;; pathΤɤˤޥåʤäϼnone-dispatch¹Ԥ롣
    ;:none-dispatch debug-cgi
    ;:none-dispatch (list nph-cgi :nph #t) ; nphˤϤ
    ;; none-dispatch̵ʤ顢404֤롣

    :errordoc-alist `((404 ,error404-cgi))

    ;; ॢ
    :request-timeout 30 ; 饤ȤHTTPإåɤ߽ФΥॢ
    ;; ʹ֤Ϥ򤹤ʤ顢礭ͤˤ롣̾5餤
    :response-timeout 3600 ; cgi¹ԻΥॢ
    :keepalive-timeout 20 ; keep-aliveॢ
    ;; reverse-proxyȤʤ顢礭ͤˤ롣
    ;; ʹ֤Ϥ򤹤ʤ顢礭ͤˤ롣̾5餤
    :use-server-header #t ; Serverإå뤫ݤǥեȤ#f
    ;:max-requests-per-child 100
    ))

;;; ----

(define (main args)
  (match args
    ((_ content-file)
     (set! *content-file* content-file)
     (load *content-file*))
    ((name . other)
     (print #`"Usage: ,name <content-file>")
     (exit 1)))
  (thread-start! (make-thread command-thunk))
  (tcpcgi-server-main *tcpcgi-server*))
